@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -47,8 +47,6 @@ struct connman_device {
 	connman_bool_t scanning;
 	connman_bool_t disconnected;
 	connman_bool_t reconnect;
-	connman_uint16_t scan_interval;
-	connman_uint16_t backoff_interval;
 	char *name;
 	char *node;
 	char *address;
@@ -58,7 +56,6 @@ struct connman_device {
 	char *devname;
 	int phyindex;
 	int index;
-	guint scan_timeout;
 	guint pending_timeout;
 
 	struct connman_device_driver *driver;
@@ -69,33 +66,6 @@ struct connman_device {
 	GHashTable *networks;
 };
 
-#define SCAN_INITIAL_DELAY 10
-
-static gboolean device_scan_trigger(gpointer user_data)
-{
-	struct connman_device *device = user_data;
-
-	DBG("device %p", device);
-
-	if (device->driver == NULL) {
-		device->scan_timeout = 0;
-		return FALSE;
-	}
-
-	if (device->driver->scan)
-		device->driver->scan(device);
-
-	return TRUE;
-}
-
-static void clear_scan_trigger(struct connman_device *device)
-{
-	if (device->scan_timeout > 0) {
-		g_source_remove(device->scan_timeout);
-		device->scan_timeout = 0;
-	}
-}
-
 static void clear_pending_trigger(struct connman_device *device)
 {
 	if (device->pending_timeout > 0) {
@@ -104,44 +74,6 @@ static void clear_pending_trigger(struct connman_device *device)
 	}
 }
 
-static void reset_scan_trigger(struct connman_device *device)
-{
-	clear_scan_trigger(device);
-
-	if (device->scan_interval > 0) {
-		guint interval;
-
-		if (g_hash_table_size(device->networks) == 0) {
-			if (device->backoff_interval >= device->scan_interval)
-				device->backoff_interval = SCAN_INITIAL_DELAY;
-			interval = device->backoff_interval;
-		} else
-			interval = device->scan_interval;
-
-		DBG("interval %d", interval);
-
-		device->scan_timeout = g_timeout_add_seconds(interval,
-					device_scan_trigger, device);
-
-		device->backoff_interval *= 2;
-		if (device->backoff_interval > device->scan_interval)
-			device->backoff_interval = device->scan_interval;
-	}
-}
-
-static void force_scan_trigger(struct connman_device *device)
-{
-	clear_scan_trigger(device);
-
-	device->scan_timeout = g_timeout_add_seconds(5,
-					device_scan_trigger, device);
-}
-
-void connman_device_schedule_scan(struct connman_device *device)
-{
-	reset_scan_trigger(device);
-}
-
 static const char *type2description(enum connman_device_type type)
 {
 	switch (type) {
@@ -304,8 +236,6 @@ int __connman_device_disable(struct connman_device *device)
 	device->powered_pending = PENDING_DISABLE;
 	device->reconnect = FALSE;
 
-	clear_scan_trigger(device);
-
 	if (device->network) {
 		struct connman_service *service =
 			__connman_service_lookup_from_network(device->network);
@@ -317,12 +247,7 @@ int __connman_device_disable(struct connman_device *device)
 	}
 
 	err = device->driver->disable(device);
-	if (err == 0) {
-		connman_device_set_powered(device, FALSE);
-		goto done;
-	}
-
-	if (err == -EALREADY) {
+	if (err == 0 || err == -EALREADY) {
 		connman_device_set_powered(device, FALSE);
 		goto done;
 	}
@@ -454,7 +379,6 @@ static void device_destruct(struct connman_device *device)
 	DBG("device %p name %s", device, device->name);
 
 	clear_pending_trigger(device);
-	clear_scan_trigger(device);
 
 	g_free(device->ident);
 	g_free(device->node);
@@ -485,7 +409,6 @@ struct connman_device *connman_device_create(const char *node,
 						enum connman_device_type type)
 {
 	struct connman_device *device;
-	connman_bool_t bg_scan;
 
 	DBG("node %s type %d", node, type);
 
@@ -497,34 +420,11 @@ struct connman_device *connman_device_create(const char *node,
 
 	device->refcount = 1;
 
-	bg_scan = connman_setting_get_bool("BackgroundScanning");
-
 	device->type = type;
 	device->name = g_strdup(type2description(device->type));
 
 	device->phyindex = -1;
 
-	device->backoff_interval = SCAN_INITIAL_DELAY;
-
-	switch (type) {
-	case CONNMAN_DEVICE_TYPE_UNKNOWN:
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-	case CONNMAN_DEVICE_TYPE_CELLULAR:
-	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_GADGET:
-	case CONNMAN_DEVICE_TYPE_VENDOR:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_WIFI:
-		if (bg_scan == TRUE)
-			device->scan_interval = 300;
-		else
-			device->scan_interval = 0;
-		break;
-	}
-
 	device->networks = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, free_network);
 
@@ -693,19 +593,16 @@ int connman_device_set_powered(struct connman_device *device,
 
 	type = __connman_device_get_service_type(device);
 
-	if (device->powered == TRUE)
-		__connman_technology_enabled(type);
-	else
+	if (device->powered == FALSE) {
 		__connman_technology_disabled(type);
-
-	if (powered == FALSE)
 		return 0;
+	}
+
+	__connman_technology_enabled(type);
 
 	connman_device_set_disconnected(device, FALSE);
 	device->scanning = FALSE;
 
-	reset_scan_trigger(device);
-
 	if (device->driver && device->driver->scan_fast)
 		device->driver->scan_fast(device);
 	else if (device->driver && device->driver->scan)
@@ -722,8 +619,6 @@ static int device_scan(struct connman_device *device)
 	if (device->powered == FALSE)
 		return -ENOLINK;
 
-	reset_scan_trigger(device);
-
 	return device->driver->scan(device);
 }
 
@@ -802,18 +697,15 @@ void __connman_device_cleanup_networks(struct connman_device *device)
 					remove_unavailable_network, NULL);
 }
 
-connman_bool_t __connman_device_scanning(struct connman_device *device)
+connman_bool_t connman_device_get_scanning(struct connman_device *device)
 {
 	return device->scanning;
 }
 
 void connman_device_reset_scanning(struct connman_device *device)
 {
-	device->scanning = FALSE;
-
 	g_hash_table_foreach(device->networks,
 				mark_network_available, NULL);
-
 }
 
 /**
@@ -839,8 +731,6 @@ int connman_device_set_scanning(struct connman_device *device,
 	if (scanning == TRUE) {
 		__connman_technology_scan_started(device);
 
-		reset_scan_trigger(device);
-
 		g_hash_table_foreach(device->networks,
 					mark_network_unavailable, NULL);
 
@@ -873,12 +763,6 @@ int connman_device_set_disconnected(struct connman_device *device,
 
 	device->disconnected = disconnected;
 
-	if (disconnected == TRUE)
-	{
-		force_scan_trigger(device);
-		device->backoff_interval = SCAN_INITIAL_DELAY;
-	}
-
 	return 0;
 }
 
@@ -1210,7 +1094,8 @@ int __connman_device_request_scan(enum connman_service_type type)
 
 int __connman_device_request_hidden_scan(struct connman_device *device,
 				const char *ssid, unsigned int ssid_len,
-				const char *identity, const char *passphrase)
+				const char *identity, const char *passphrase,
+				void *user_data)
 {
 	DBG("device %p", device);
 
@@ -1222,12 +1107,13 @@ int __connman_device_request_hidden_scan(struct connman_device *device,
 		return -EALREADY;
 
 	return device->driver->scan_hidden(device, ssid, ssid_len,
-					identity, passphrase);
+					identity, passphrase, user_data);
 }
 
 connman_bool_t __connman_device_isfiltered(const char *devname)
 {
 	char **pattern;
+	char **blacklisted_interfaces;
 
 	if (device_filter == NULL)
 		goto nodevice;
@@ -1246,7 +1132,7 @@ nodevice:
 	}
 
 	if (nodevice_filter == NULL)
-		return FALSE;
+		goto list;
 
 	for (pattern = nodevice_filter; *pattern; pattern++) {
 		if (g_pattern_match_simple(*pattern, devname) == TRUE) {
@@ -1255,6 +1141,19 @@ nodevice:
 		}
 	}
 
+list:
+	blacklisted_interfaces =
+		connman_setting_get_string_list("NetworkInterfaceBlacklist");
+	if (blacklisted_interfaces == NULL)
+		return FALSE;
+
+	for (pattern = blacklisted_interfaces; *pattern; pattern++) {
+		if (g_str_has_prefix(devname, *pattern) == TRUE) {
+			DBG("ignoring device %s (blacklist)", devname);
+			return TRUE;
+		}
+	}
+
 	return FALSE;
 }
 
