@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -58,6 +58,9 @@ struct connman_provider {
 	struct connman_provider_driver *driver;
 	void *driver_data;
 	GHashTable *setting_strings;
+	GHashTable *user_routes;
+	gchar **user_networks;
+	gsize num_user_networks;
 };
 
 void __connman_provider_append_properties(struct connman_provider *provider,
@@ -76,6 +79,93 @@ void __connman_provider_append_properties(struct connman_provider *provider,
 						 &provider->type);
 }
 
+int __connman_provider_append_user_route(struct connman_provider *provider,
+			int family, const char *network, const char *netmask)
+{
+	struct connman_route *route;
+	char *key = g_strdup_printf("%d/%s/%s", family, network, netmask);
+
+	DBG("family %d network %s netmask %s", family, network, netmask);
+
+	route = g_hash_table_lookup(provider->user_routes, key);
+	if (route == NULL) {
+		route = g_try_new0(struct connman_route, 1);
+		if (route == NULL) {
+			connman_error("out of memory");
+			return -ENOMEM;
+		}
+
+		route->family = family;
+		route->host = g_strdup(network);
+		route->netmask = g_strdup(netmask);
+
+		g_hash_table_replace(provider->user_routes, key, route);
+	} else
+		g_free(key);
+
+	return 0;
+}
+
+static void set_user_networks(struct connman_provider *provider,
+							char **networks)
+{
+	int i = 0;
+
+	while (networks[i] != NULL) {
+		char **elems = g_strsplit(networks[i], "/", 0);
+		char *network, *netmask;
+		int family = PF_UNSPEC, ret;
+
+		if (elems == NULL)
+			break;
+
+		network = elems[0];
+		if (network == NULL || *network == '\0') {
+			DBG("no network/netmask set");
+			g_strfreev(elems);
+			break;
+		}
+
+		netmask = elems[1];
+		if (netmask != NULL && *netmask == '\0') {
+			DBG("no netmask set");
+			g_strfreev(elems);
+			break;
+		}
+
+		if (g_strrstr(network, ":") != NULL)
+			family = AF_INET6;
+		else if (g_strrstr(network, ".") != NULL) {
+			family = AF_INET;
+
+			if (g_strrstr(netmask, ".") == NULL) {
+				/* We have netmask length */
+				in_addr_t addr;
+				struct in_addr netmask_in;
+				unsigned char prefix_len = 32;
+
+				if (netmask != NULL)
+					prefix_len = atoi(netmask);
+
+				addr = 0xffffffff << (32 - prefix_len);
+				netmask_in.s_addr = htonl(addr);
+				netmask = inet_ntoa(netmask_in);
+
+				DBG("network %s netmask %s", network, netmask);
+			}
+		}
+
+		ret = __connman_provider_append_user_route(provider,
+						family, network, netmask);
+		g_strfreev(elems);
+
+		if (ret != 0)
+			break;
+
+		i++;
+	}
+}
+
 static int provider_load_from_keyfile(struct connman_provider *provider,
 		GKeyFile *keyfile)
 {
@@ -94,16 +184,30 @@ static int provider_load_from_keyfile(struct connman_provider *provider,
 	while (idx < length) {
 		key = settings[idx];
 		if (key != NULL) {
-			value = g_key_file_get_string(keyfile,
+			if (g_str_equal(key, "Networks") == TRUE) {
+				g_strfreev(provider->user_networks);
+				provider->user_networks =
+					g_key_file_get_string_list(keyfile,
 						provider->identifier,
-						key, NULL);
-			connman_provider_set_string(provider, key, value);
-			g_free(value);
+						key,
+						&provider->num_user_networks,
+						NULL);
+			} else {
+				value = g_key_file_get_string(keyfile,
+							provider->identifier,
+							key, NULL);
+				connman_provider_set_string(provider, key,
+							value);
+				g_free(value);
+			}
 		}
 		idx += 1;
 	}
 	g_strfreev(settings);
 
+	if (provider->user_networks != NULL)
+		set_user_networks(provider, provider->user_networks);
+
 	return 0;
 }
 
@@ -141,6 +245,11 @@ static int connman_provider_save(struct connman_provider *provider)
 			"Host", provider->host);
 	g_key_file_set_string(keyfile, provider->identifier,
 			"VPN.Domain", provider->domain);
+	if (provider->user_networks != NULL)
+		g_key_file_set_string_list(keyfile, provider->identifier,
+				"Networks",
+				(const gchar **)provider->user_networks,
+				provider->num_user_networks);
 
 	if (provider->driver != NULL && provider->driver->save != NULL)
 		provider->driver->save(provider, keyfile);
@@ -237,8 +346,11 @@ static void provider_destruct(struct connman_provider *provider)
 	g_free(provider->host);
 	g_free(provider->domain);
 	g_free(provider->identifier);
+	g_strfreev(provider->user_networks);
 	g_hash_table_destroy(provider->routes);
+	g_hash_table_destroy(provider->user_routes);
 	g_hash_table_destroy(provider->setting_strings);
+	g_free(provider);
 }
 
 void connman_provider_unref_debug(struct connman_provider *provider,
@@ -394,6 +506,9 @@ static int set_connected(struct connman_provider *provider,
 		g_hash_table_foreach(provider->routes, provider_append_routes,
 					provider);
 
+		g_hash_table_foreach(provider->user_routes, provider_append_routes,
+					provider);
+
 	} else {
 		if (ipconfig != NULL) {
 			provider_indicate_state(provider,
@@ -492,8 +607,11 @@ static void provider_initialize(struct connman_provider *provider)
 	provider->type = NULL;
 	provider->domain = NULL;
 	provider->identifier = NULL;
+	provider->user_networks = NULL;
 	provider->routes = g_hash_table_new_full(g_direct_hash, g_direct_equal,
 					NULL, destroy_route);
+	provider->user_routes = g_hash_table_new_full(g_str_hash, g_str_equal,
+					g_free, destroy_route);
 	provider->setting_strings = g_hash_table_new_full(g_str_hash, g_str_equal,
 							g_free, g_free);
 }
@@ -641,14 +759,52 @@ static void provider_create_all_from_type(const char *provider_type)
 	g_strfreev(providers);
 }
 
+static char **get_user_networks(DBusMessageIter *array, int *count)
+{
+	DBusMessageIter entry;
+	char **networks = NULL;
+	GSList *list = NULL, *l;
+	int len;
+
+	dbus_message_iter_recurse(array, &entry);
+
+	while (dbus_message_iter_get_arg_type(&entry) == DBUS_TYPE_STRING) {
+		const char *val;
+		dbus_message_iter_get_basic(&entry, &val);
+
+		list = g_slist_prepend(list, g_strdup(val));
+		dbus_message_iter_next(&entry);
+	}
+
+	len = g_slist_length(list);
+	if (len == 0)
+		goto out;
+
+	networks = g_try_new(char *, len + 1);
+	if (networks == NULL)
+		goto out;
+
+	*count = len;
+	networks[len] = 0;
+
+	for (l = list; l != NULL; l = g_slist_next(l))
+		networks[--len] = l->data;
+
+out:
+	g_slist_free(list);
+
+	return networks;
+}
+
 int __connman_provider_create_and_connect(DBusMessage *msg)
 {
 	struct connman_provider *provider;
 	DBusMessageIter iter, array;
 	const char *type = NULL, *name = NULL, *service_path;
 	const char *host = NULL, *domain = NULL;
+	char **networks = NULL;
 	char *ident;
-	int err;
+	int err, count = 0;
 
 	dbus_message_iter_init(msg, &iter);
 	dbus_message_iter_recurse(&iter, &array);
@@ -674,6 +830,10 @@ int __connman_provider_create_and_connect(DBusMessage *msg)
 			else if (g_str_equal(key, "VPN.Domain") == TRUE)
 				dbus_message_iter_get_basic(&value, &domain);
 			break;
+		case DBUS_TYPE_ARRAY:
+			if (g_str_equal(key, "Networks") == TRUE)
+				networks = get_user_networks(&value, &count);
+			break;
 		}
 
 		dbus_message_iter_next(&array);
@@ -682,7 +842,7 @@ int __connman_provider_create_and_connect(DBusMessage *msg)
 	if (host == NULL || domain == NULL)
 		return -EINVAL;
 
-	DBG("Type %s name %s", type, name);
+	DBG("Type %s name %s networks %p", type, name, networks);
 
 	if (type == NULL || name == NULL)
 		return -EOPNOTSUPP;
@@ -710,6 +870,13 @@ int __connman_provider_create_and_connect(DBusMessage *msg)
 			connman_provider_load(provider);
 	}
 
+	if (networks != NULL) {
+		g_strfreev(provider->user_networks);
+		provider->user_networks = networks;
+		provider->num_user_networks = count;
+		set_user_networks(provider, provider->user_networks);
+	}
+
 	dbus_message_iter_init(msg, &iter);
 	dbus_message_iter_recurse(&iter, &array);
 
@@ -816,6 +983,23 @@ const char *connman_provider_get_string(struct connman_provider *provider,
 	return g_hash_table_lookup(provider->setting_strings, key);
 }
 
+connman_bool_t
+__connman_provider_check_routes(struct connman_provider *provider)
+{
+	if (provider == NULL)
+		return FALSE;
+
+	if (provider->user_routes != NULL &&
+			g_hash_table_size(provider->user_routes) > 0)
+		return TRUE;
+
+	if (provider->routes != NULL &&
+			g_hash_table_size(provider->routes) > 0)
+		return TRUE;
+
+	return FALSE;
+}
+
 void *connman_provider_get_data(struct connman_provider *provider)
 {
 	return provider->driver_data;
@@ -1110,9 +1294,71 @@ static void provider_offline_mode(connman_bool_t enabled)
 
 }
 
+static struct connman_provider *provider_get(int index)
+{
+	GHashTableIter iter;
+	gpointer value, key;
+
+	g_hash_table_iter_init(&iter, provider_hash);
+
+	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
+		struct connman_provider *provider = value;
+
+		if (provider->index == index)
+			return provider;
+	}
+
+	return NULL;
+}
+
+static void provider_service_changed(struct connman_service *service,
+				enum connman_service_state state)
+{
+	struct connman_provider *provider;
+	int vpn_index, service_index;
+
+	if (service == NULL)
+		return;
+
+	switch (state) {
+	case CONNMAN_SERVICE_STATE_UNKNOWN:
+	case CONNMAN_SERVICE_STATE_IDLE:
+	case CONNMAN_SERVICE_STATE_ASSOCIATION:
+	case CONNMAN_SERVICE_STATE_CONFIGURATION:
+	case CONNMAN_SERVICE_STATE_READY:
+	case CONNMAN_SERVICE_STATE_ONLINE:
+		return;
+	case CONNMAN_SERVICE_STATE_DISCONNECT:
+	case CONNMAN_SERVICE_STATE_FAILURE:
+		break;
+	}
+
+	service_index = __connman_service_get_index(service);
+
+	vpn_index = __connman_connection_get_vpn_index(service_index);
+
+	DBG("service %p %s state %d index %d/%d", service,
+		__connman_service_get_ident(service),
+		state, service_index, vpn_index);
+
+	if (vpn_index < 0)
+		return;
+
+	provider = provider_get(vpn_index);
+	if (provider == NULL)
+		return;
+
+	DBG("disconnect %p index %d", provider, vpn_index);
+
+	__connman_provider_disconnect(provider);
+
+	return;
+}
+
 static struct connman_notifier provider_notifier = {
 	.name			= "provider",
 	.offline_mode		= provider_offline_mode,
+	.service_state_changed	= provider_service_changed,
 };
 
 int __connman_provider_init(void)
